@@ -57,6 +57,9 @@
  *
  * -- Revision History 
  * $Log: gif.c,v $
+ * Revision 1.2  2003/09/01 16:23:31  beazley
+ * Restored the 'mojo'.
+ *
  * Revision 1.2  1996/09/25 22:39:30  dmb
  * Fixed prototypes and use of void pointers for compatibility with the Cray T3D
  *
@@ -407,8 +410,6 @@ static void compress(init_bits, data, len)
     }
     len--;
 
-#ifdef ALLOW_LZW
-
     fcode = (long) ( ( (long) c << maxbits) + ent);
        i = (((int) c << hshift) ^ ent);    /* xor hashing */
 
@@ -448,17 +449,6 @@ nomatch:
     else
       cl_block();
 
-#else
-    /* Uncompressed GIF */
-    output_GIF(ent);
-    code_count++;
-    if (code_count >= ClearCode-2) {
-      output_GIF(ClearCode);
-      code_count = 0;
-    }
-    ent = c;
-#endif
-
   }
   /* Put out the final code */
   output_GIF(ent);
