@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.11 2004/11/15 20:51:47 marcelomatus Exp $";
+char cvsroot_cscanner_c[] = "$Header: /cvsroot/SWIG/Source/CParse/cscanner.c,v 1.6 2003/08/11 17:55:04 beazley Exp $";
 
 #include "cparse.h"
 #include "parser.h"
@@ -47,7 +47,7 @@ static  int    num_brace = 0;
 static  int    last_brace = 0;
 static  int    last_id = 0;
 static  int    rename_active = 0;
-        int    cparse_cplusplus = 0;
+        int    cparse_cplusplus;
 
 /* -----------------------------------------------------------------------------
  * Swig_cparse_cplusplus()
@@ -269,10 +269,9 @@ void start_inline(char *text, int line) {
  * Inserts a comment into a documentation entry.
  **************************************************************/
 
-/*
 void yycomment(char *a, int b, int c) {
 }
-*/
+
 
 /* -----------------------------------------------------------------------------
  * skip_balanced()
@@ -1188,7 +1187,7 @@ int yylex(void) {
 	      return(TEMPLATE);
 	    }
 	    if (strcmp(yytext,"delete") == 0) {
-	      return(DELETE_KW);
+	      return(DELETE);
 	    }
 	    if (strcmp(yytext,"using") == 0) {
 	      return(USING);
@@ -1200,10 +1199,6 @@ int yylex(void) {
 	    if (strcmp(yytext,"class") == 0) {
 	      Swig_warning(WARN_PARSE_CLASS_KEYWORD,cparse_file,cparse_line, "class keyword used, but not in C++ mode.\n");
 	    }
-	    if (strcmp(yytext,"complex") == 0) {
-	      yylval.type = NewSwigType(T_COMPLEX);
-	      return(TYPE_COMPLEX);
-	    }
 	  }
 	  
 	  /* Objective-C keywords */
@@ -1223,7 +1218,7 @@ int yylex(void) {
 	  /* Misc keywords */
 	  
 	  if (strcmp(yytext,"extern") == 0) return(EXTERN);
-	  if (strcmp(yytext,"const") == 0) return(CONST_QUAL);
+	  if (strcmp(yytext,"const") == 0) return(CONST);
 	  if (strcmp(yytext,"static") == 0) return(STATIC);
 	  if (strcmp(yytext,"struct") == 0) return(STRUCT);
 	  if (strcmp(yytext,"union") == 0) return(UNION);
@@ -1238,7 +1233,6 @@ int yylex(void) {
 	  /* Ignored keywords */
 	  
 	  if (strcmp(yytext,"volatile") == 0) return(VOLATILE);
-	  if (strcmp(yytext,"register") == 0) return(REGISTER);
 	  
 	  /* SWIG directives */
 	} else {
