@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v 1.49 2004/11/22 01:37:29 marcelomatus Exp $";
+char cvsroot_main_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/main.cxx,v 1.27 2003/12/16 03:43:51 beazley Exp $";
 
 #if defined(_WIN32)
 #define WIN32_LEAN_AND_MEAN
@@ -20,61 +20,55 @@ char cvsroot_main_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/main.cxx,v
 #endif
 
 #include "swigmod.h"
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
 
 #include "swigwarn.h"
-#include "cparse.h"
+
+extern "C" {
+#include "preprocessor.h"
+}
+
 #include <ctype.h>
 
 // Global variables
 
-    char       LibDir[512];                     // Library directory
+    char       LibDir[512];                      // Library directory
     Language  *lang;                            // Language method
     int        CPlusPlus = 0;
     int        Extend = 0;                      // Extend flag
     int        ForceExtern = 0;                 // Force extern mode
     int        GenerateDefault = 1;             // Generate default constructors
     char      *Config = 0;
+    int        NoInclude = 0;
     int        Verbose = 0;
     int        NoExtern = 0;
     int        NoExcept = 0;
     char      *SwigLib;
-    int        SwigRuntime = 0;                 // 0 = no option, 1 = -c or -runtime, 2 = -noruntime
 
 
 extern "C" {
 extern String  *ModuleName;
 }
 
-static const char *usage1 = (const char*)"\
+static char *usage = (char*)"\
 \nGeneral Options\n\
      -c++            - Enable C++ processing\n\
-     -co <file>      - Check <file> out of the SWIG library\n\
-     -directors      - Turn on director mode for all the classes, mainly for testing \n\
+     -co             - Check a file out of the SWIG library\n\
      -dirprot        - Turn on wrapping of protected members for director classes\n\
      -D<symbol>      - Define a symbol <symbol> (for conditional compilation)\n\
      -E              - Preprocess only, does not generate wrapper code\n\
      -fcompact       - Compile in compact mode\n\
-     -features list  - Set a list of global features, where the syntax list is \n\
-                       -features directors,autodoc=1      \n\
-                       if not explicit value is given to the feature, a '1' is used \n\
      -fvirtual       - Compile in virtual elimination mode\n\
-     -Fstandard      - Display error/warning messages in commonly used format\n\
-     -Fmicrosoft     - Display error/warning messages in Microsoft format\n\
      -help           - This output\n\
      -I<dir>         - Look for SWIG files in <dir>\n\
-     -I-             - Don't search the current directory\n\
      -ignoremissing  - Ignore missing include files\n\
      -importall      - Follow all #include statements as imports\n\
      -includeall     - Follow all #include statements\n\
      -l<ifile>       - Include SWIG library file <ifile>\n\
      -M              - List all dependencies \n\
-     -MD             - Is equivalent to `-M -MF <file>', except `-E' is not implied\n\
-     -MF <file>      - Generate dependencies into <file> and continue generating wrappers \n\
      -MM             - List dependencies, but omit files in SWIG library\n\
-     -MMD            - Like `-MD', but omit files in SWIG library\n\
-";
-// usage string split in two otherwise string is too big for some compilers
-static const char *usage2 = (const char*)"\
      -makedefault    - Create default constructors/destructors (the default)\n\
      -module <name>  - Set module name to <name>\n\
      -nocontract     - Turn off contract checking \n\
@@ -82,52 +76,25 @@ static const char *usage2 = (const char*)"\
      -nodirprot      - Do not wrap director protected members\n\
      -noexcept       - Do not wrap exception specifiers\n\
      -noextern       - Do not generate extern declarations\n\
-     -nopreprocess   - Skip the preprocessor step\n\
+     -noruntime      - Do not include SWIG runtime code\n\
      -o <outfile>    - Set name of the output file to <outfile>\n\
-     -oh <headfile>  - Set name of the output header file to <headfile>\n\
      -outdir <dir>   - Set language specific files output directory\n\
+     -runtime        - Make the runtime support code globally visible.\n\
      -small          - Compile in virtual elimination & compact mode\n\
      -swiglib        - Report location of SWIG library and exit\n\
-     -templatereduce - Reduce all the typedefs in templates \n\
      -v              - Run in verbose mode\n\
      -version        - Print SWIG version number\n\
      -Wall           - Enable all warning messages\n\
-     -Wallkw         - Enable keyword warnings for all the supported languages\n\
-     -Werror         - Treat warnings as errors\n\
-     -w<list>        - Suppress/add warning messages eg -w401,+321 - see Warnings.html\n\
-     -xmlout <file>  - Write XML version of the parse tree to <file> after normal processing\n\
+     -Wallkw         - Enable warning messages for all the languages keywords\n\
+     -Werror         - Force to treat the warnings as errors\n\
+     -w<n>           - Suppress warning number <n>\n\
 \n";
 
 // Local variables
 static int     freeze = 0;
 static String  *lang_config = 0;
-static char    *hpp_extension = (char *) "h";
 static char    *cpp_extension = (char *) "cxx";
-static char    *depends_extension = (char *) "d";
 static String  *outdir = 0;
-static String  *xmlout = 0;
-static int     help = 0;
-static int     checkout = 0;
-static int     cpp_only = 0;
-static int     no_cpp = 0;
-static char   *outfile_name = 0;
-static char   *outfile_name_h = 0;
-static int     tm_debug = 0;
-static int     dump_tags = 0;
-static int     dump_tree = 0;
-static int     dump_xml = 0;
-static int     browse = 0;
-static int     dump_typedef = 0;
-static int     dump_classes = 0;
-static int     werror = 0;
-static int     depend = 0;
-static int     depend_only = 0;
-static int     memory_debug = 0;
-static int     allkw = 0;
-static DOH    *libfiles = 0;
-static DOH    *cpps = 0 ;
-static String  *dependencies_file = 0;
-static File    *f_dependencies_file = 0;
 
 // -----------------------------------------------------------------------------
 // check_suffix(char *name)
@@ -135,7 +102,8 @@ static File    *f_dependencies_file = 0;
 // Checks the suffix of a file to see if we should emit extern declarations.
 // -----------------------------------------------------------------------------
 
-static int check_suffix(char *name) {
+int
+check_suffix(char *name) {
   char *c;
   if (!name) return 0;
   c = Swig_file_suffix(name);
@@ -155,7 +123,8 @@ static int check_suffix(char *name) {
 // Install all command line options as preprocessor symbols
 // ----------------------------------------------------------------------------- 
 
-static void install_opts(int argc, char *argv[]) {
+static void
+install_opts(int argc, char *argv[]) {
   int i;
   int noopt = 0;
   char *c;
@@ -232,330 +201,52 @@ void SWIG_config_cppext(const char *ext) {
   cpp_extension = (char *) ext;
 }
 
-void SWIG_getfeatures(const char *c) 
-{
-  char feature[64];
-  char *fb = feature;
-  char *fe = fb + 63;
-  Hash *features_hash = Swig_cparse_features();
-  String *name = NewString("");
-  /* Printf(stderr,"all features %s\n", c); */
-  while (*c) {
-    char *f = fb;
-    String *fname = NewString("feature:");
-    String *fvalue = NewString("");
-    while ((f != fe) && *c != '=' && *c != ',' && *c) {
-      *(f++) = *(c++);
-    }
-    *f = 0;
-    Printf(fname,"%s",feature);
-    if (*c && *(c++) == '=') {
-      char value[64];
-      char *v = value;
-      char *ve = v + 63;
-      while ((v != ve) &&  *c != ',' && *c && !isspace(*c)) {
-	*(v++) = *(c++);
-      }
-      *v = 0;
-      Printf(fvalue,"%s",value);
-    } else {
-      Printf(fvalue,"1");
-    }    
-    /* Printf(stderr,"%s %s\n", fname, fvalue);  */
-    Swig_feature_set(features_hash,name,0,fname,fvalue,0);
-    Delete(fname);
-    Delete(fvalue);
-  }
-  Delete(name);
-}
-
-void SWIG_getoptions(int argc, char *argv[]) 
-{
-  int    i;
-  char   *includefiles[256];
-  int     includecount = 0;
-  // Get options
-  for (i = 1; i < argc; i++) {
-      if (argv[i]) {
-	  if (strncmp(argv[i],"-I-",3) == 0) {
-	    // Don't push/pop directories
-	    Swig_set_push_dir(0);
-	    Swig_mark_arg(i);
-	  } else if (strncmp(argv[i],"-I",2) == 0) {
-	    // Add a new directory search path
-	    includefiles[includecount++] = Swig_copy_string(argv[i]+2);
-	    Swig_mark_arg(i);
-	  } else if (strncmp(argv[i],"-D",2) == 0) {
-	    DOH *d = NewString(argv[i]+2);
-	    Replace(d,(char*)"=",(char*)" ", DOH_REPLACE_ANY | DOH_REPLACE_FIRST);
-	    Preprocessor_define((DOH *) d,0);
-	    // Create a symbol
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-E") == 0) {
-	    cpp_only = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-nopreprocess") == 0) {
-	    no_cpp = 1;
-	    Swig_mark_arg(i);
-	  } else if ((strcmp(argv[i],"-verbose") == 0) ||
-		     (strcmp(argv[i],"-v") == 0)) {
-	      Verbose = 1;
-	      Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-c++") == 0) {
-	      CPlusPlus=1;
-	      Preprocessor_define((DOH *) "__cplusplus __cplusplus", 0);
-	      Swig_cparse_cplusplus(1);
-	      Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-fcompact") == 0) {
-	    Wrapper_compact_print_mode_set(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-fvirtual") == 0) {
-	    Wrapper_virtual_elimination_mode_set(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-directors") == 0) {
-	    Hash *features_hash = Swig_cparse_features();
-	    String *name = NewString("");
-	    String *fname = NewString("feature:director");
-	    String *fvalue = NewString("1");
-	    Swig_feature_set(features_hash,name,0,fname,fvalue,0);
-	    Wrapper_director_mode_set(1);
-	    Swig_mark_arg(i);
-	    Delete(name);
-	    Delete(fname);
-	    Delete(fvalue);
-	  } else if (strcmp(argv[i],"-dirprot") == 0) {
-	    Wrapper_director_protected_mode_set(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-nodirprot") == 0) {
-	    Wrapper_director_protected_mode_set(0);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-small") == 0) {
-	    Wrapper_compact_print_mode_set(1);
-	    Wrapper_virtual_elimination_mode_set(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i], "-runtime") == 0) {
-	      Swig_mark_arg(i);
-	      Swig_warning(WARN_DEPRECATED_OPTC, "SWIG",1, "-c, -runtime, -noruntime command line options are deprecated.\n");
-	      SwigRuntime = 1;
-	  } else if ((strcmp(argv[i],"-c") == 0) || (strcmp(argv[i],"-noruntime") == 0)) {
-	      Swig_mark_arg(i);
-	      Swig_warning(WARN_DEPRECATED_OPTC, "SWIG",1, "-c, -runtime, -noruntime command line options are deprecated.\n");
-	      SwigRuntime = 2;
-          } else if ((strcmp(argv[i],"-make_default") == 0) || (strcmp(argv[i],"-makedefault") == 0)) {
-	    GenerateDefault = 1;
-	    Swig_mark_arg(i);
-          } else if ((strcmp(argv[i],"-no_default") == 0) || (strcmp(argv[i],"-nodefault") == 0)) {
-	    GenerateDefault = 0;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-noexcept") == 0) {
-	    NoExcept = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-noextern") == 0) {
-	    NoExtern = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-show_templates") == 0) {
-	    Swig_cparse_debug_templates(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-templatereduce") == 0) {
-	    SWIG_cparse_template_reduce(1);
-	    Swig_mark_arg(i);
-          } else if (strcmp(argv[i],"-swiglib") == 0) {
-	    printf("%s\n", LibDir);
-	    SWIG_exit (EXIT_SUCCESS);
-	  } else if (strcmp(argv[i],"-o") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      outfile_name = Swig_copy_string(argv[i+1]);
-	      if (!outfile_name_h || !dependencies_file) {
-		char *ext = strrchr(outfile_name, '.');
-		String *basename = ext ? NewStringWithSize(outfile_name,ext-outfile_name) : NewString(outfile_name);
-                if (!dependencies_file) {
-                  dependencies_file = NewStringf("%s.%s", basename, depends_extension);
-                }
-                if (!outfile_name_h) {
-                  Printf(basename, ".%s", hpp_extension);
-                  outfile_name_h = Swig_copy_string(Char(basename));
-                  Delete(basename);
-                }
-	      }
-	      Swig_mark_arg(i+1);
-	      i++;
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-oh") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      outfile_name_h = Swig_copy_string(argv[i+1]);
-	      Swig_mark_arg(i+1);
-	      i++;
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-version") == 0) {
- 	      fprintf(stderr,"\nSWIG Version %s\n", PACKAGE_VERSION);
-	      fprintf(stderr,"Copyright (c) 1995-1998\n");
-	      fprintf(stderr,"University of Utah and the Regents of the University of California\n");
-              fprintf(stderr,"Copyright (c) 1998-2004\n");
-	      fprintf(stderr,"University of Chicago\n");
-	      fprintf(stderr,"Compiled with %s [%s]\n", SWIG_CXX, SWIG_PLATFORM);
-	      fprintf(stderr,"\nPlease see %s for reporting bugs and further information\n", PACKAGE_BUGREPORT);
-	      SWIG_exit (EXIT_SUCCESS);
-	  } else if (strncmp(argv[i],"-l",2) == 0) {
-	    // Add a new directory search path
-	    Append(libfiles,argv[i]+2);
-	    Swig_mark_arg(i);
-          } else if (strcmp(argv[i],"-co") == 0) {
-	    checkout = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-features") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      SWIG_getfeatures(argv[i+1]);
-	      Swig_mark_arg(i+1);
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-freeze") == 0) {
-	    freeze = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-includeall") == 0) {
-	    Preprocessor_include_all(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-importall") == 0) {
-	    Preprocessor_import_all(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-ignoremissing") == 0) {
-	    Preprocessor_ignore_missing(1);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-tm_debug") == 0) {
-	    tm_debug = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-module") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      ModuleName = NewString(argv[i+1]);	    
-	      Swig_mark_arg(i+1);
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-M") == 0) {
-	    depend = 1;
-	    depend_only = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-MM") == 0) {
-	    depend = 2;
-	    depend_only = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-MF") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      dependencies_file = NewString(argv[i+1]);	    
-	      Swig_mark_arg(i+1);
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-MD") == 0) {
-	    depend = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-MMD") == 0) {
-	    depend = 2;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-outdir") == 0) {
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      outdir = NewString(argv[i+1]);	    
-	      Swig_mark_arg(i+1);
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-Wall") == 0) {
-	    Swig_mark_arg(i);
-	    Swig_warnall();
-	  } else if (strcmp(argv[i],"-Wallkw") == 0) {
-	    allkw = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-Werror") == 0) {
-	    werror = 1;
-	    Swig_mark_arg(i);
-	  } else if (strncmp(argv[i],"-w",2) == 0) {
-	    Swig_mark_arg(i);
-	    Swig_warnfilter(argv[i]+2,1);
-	  } else if (strcmp(argv[i],"-dump_tags") == 0) {
-	    dump_tags = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_tree") == 0) {
-	    dump_tree = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_xml") == 0) {
-	    dump_xml = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-xmlout") == 0) {
-	    dump_xml = 1;
-	    Swig_mark_arg(i);
-	    if (argv[i+1]) {
-	      xmlout = NewString(argv[i+1]);	    
-	      Swig_mark_arg(i+1);
-	    } else {
-	      Swig_arg_error();
-	    }
-	  } else if (strcmp(argv[i],"-nocontract") == 0) {
-	    Swig_mark_arg(i);
-	    Swig_contract_mode_set(0);
-	  } else if (strcmp(argv[i],"-browse") == 0) {
-	    browse = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_typedef") == 0) {
-	    dump_typedef = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_classes") == 0) {
-	    dump_classes = 1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-dump_memory") == 0) {
-	    memory_debug =1;
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-Fstandard") == 0) {
-            Swig_error_msg_format(EMF_STANDARD);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-Fmicrosoft") == 0) {
-            Swig_error_msg_format(EMF_MICROSOFT);
-	    Swig_mark_arg(i);
-	  } else if (strcmp(argv[i],"-help") == 0) {
-	    fputs(usage1,stderr);
-	    fputs(usage2,stderr);
-	    Swig_mark_arg(i);
-	    help = 1;
-	  }
-      }
-  }
-
-  for (i = 0; i < includecount; i++) {
-    Swig_add_directory((DOH *) includefiles[i]);
-  }
-}
-
-
-
+extern  "C" Node *Swig_cparse(File *);
+extern  "C" void  Swig_cparse_cplusplus(int);
+extern  "C" void  Swig_cparse_debug_templates(int);
+extern  void Wrapper_virtual_elimination_mode_set(int);
+extern  void Wrapper_director_protected_mode_set(int);
 
+extern void Swig_contracts(Node *n);
+extern void Swig_contract_mode_set(int flag);
+extern int  Swig_contract_mode_get();
 
 int SWIG_main(int argc, char *argv[], Language *l) {
+  int    i;
   char   *c;
   char    temp[512];
+  char   *outfile_name = 0;
+  int     help = 0;
+  int     checkout = 0;
+  int     cpp_only = 0;
+  int     tm_debug = 0;
+  char   *includefiles[256];
+  int     includecount = 0;
+  extern  int check_suffix(char *);
+  int     dump_tags = 0;
+  int     dump_tree = 0;
+  int     browse = 0;
+  int     dump_typedef = 0;
+  int     dump_classes = 0;
+  int     werror = 0;
+  int     depend = 0;
+  int     memory_debug = 0;
+  int     allkw = 0;
+
+  DOH    *libfiles = 0;
+  DOH    *cpps = 0 ;
+  extern void Swig_browser(Node *n, int);
+  extern void Swig_default_allocators(Node *n);
+  extern void Swig_process_types(Node *n);
+
 
   /* Initialize the SWIG core */
   Swig_init();
   
-  /* Suppress warning messages for private inheritance, preprocessor
-     evaluation, might be abstract, overloaded const, and ...
+  /* Suppress warning messages for private inheritance, preprocessor evaluation, 
+     might be abstract, and overloaded const */
 
-     WARN_PP_EVALUATION            202
-     WARN_PARSE_PRIVATE_INHERIT    309
-     WARN_TYPE_ABSTRACT            403
-     WARN_LANG_OVERLOAD_CONST      512
-     WARN_PARSE_BUILTIN_NAME       321
-     WARN_PARSE_REDUNDANT          322
-  */
-  Swig_warnfilter("202,309,403,512,321,322",1);
+  Swig_warnfilter("202,309,403,512,321",1);
 
   // Initialize the preprocessor
   Preprocessor_init();
@@ -594,13 +285,9 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   Swig_contract_mode_set(1);
   Preprocessor_define(vers,0);
 
-  /* Turn off directors mode */
-  Wrapper_director_mode_set(0);
+  /* Turn on director protected mode */
   Wrapper_director_protected_mode_set(0);
 
-  /* Turn off template extmode */
-  Wrapper_template_extmode_set(0);
-
 
   // Check for SWIG_LIB environment variable
 
@@ -628,12 +315,212 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   libfiles = NewList();
 
   /* Check for SWIG_FEATURES environment variable */
+  if ((c = getenv("SWIG_FEATURES"))) {
+    while (*c!='\0') {
+      while ((*c)==' ') {
+	c++;
+      }
+      i = 0;
+      while ((*c!='\0') && (*c!=' ')) {
+	temp[i] = *c;
+	c++; i++;
+      }
+      temp[i]='\0';
+      
+      if (strcmp(temp, "-fcompact") == 0) {
+	Wrapper_compact_print_mode_set(1);
+      } else if (strcmp(temp, "-fvirtual") == 0) {
+	Wrapper_virtual_elimination_mode_set(1);
+      } else if (strcmp(temp,"-dirprot") == 0) {
+	  Wrapper_director_protected_mode_set(1);
+      } else if (strcmp(temp,"-nodirprot") == 0) {
+	  Wrapper_director_protected_mode_set(0);
+      } else if (strcmp(temp, "-small") == 0) {
+	Wrapper_compact_print_mode_set(1);
+	Wrapper_virtual_elimination_mode_set(1);
+      }
+    }
+  }
+  
+  // Get options
+  for (i = 1; i < argc; i++) {
+      if (argv[i]) {
+	  if (strncmp(argv[i],"-I",2) == 0) {
+	    // Add a new directory search path
+	    includefiles[includecount++] = Swig_copy_string(argv[i]+2);
+	    Swig_mark_arg(i);
+	  } else if (strncmp(argv[i],"-D",2) == 0) {
+	    DOH *d = NewString(argv[i]+2);
+	    Replace(d,(char*)"=",(char*)" ", DOH_REPLACE_ANY | DOH_REPLACE_FIRST);
+	    Preprocessor_define((DOH *) d,0);
+	    // Create a symbol
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-E") == 0) {
+	    cpp_only = 1;
+	    Swig_mark_arg(i);
+	  } else if ((strcmp(argv[i],"-verbose") == 0) ||
+		     (strcmp(argv[i],"-v") == 0)) {
+	      Verbose = 1;
+	      Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-c++") == 0) {
+	      CPlusPlus=1;
+	      Preprocessor_define((DOH *) "__cplusplus 1", 0);
+	      Swig_cparse_cplusplus(1);
+	      Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-fcompact") == 0) {
+	    Wrapper_compact_print_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-fvirtual") == 0) {
+	    Wrapper_virtual_elimination_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dirprot") == 0) {
+	    Wrapper_director_protected_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-nodirprot") == 0) {
+	    Wrapper_director_protected_mode_set(0);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-small") == 0) {
+	    Wrapper_compact_print_mode_set(1);
+	    Wrapper_virtual_elimination_mode_set(1);
+	    Swig_mark_arg(i);
+	  } else if ((strcmp(argv[i],"-noruntime") == 0) || (strcmp(argv[i],"-c") == 0)) {
+	      NoInclude=1;
+	      Preprocessor_define((DOH *) "SWIG_NOINCLUDE 1", 0);
+	      Swig_mark_arg(i);
+	      if (strcmp(argv[i],"-c") == 0) {
+		Swig_warning(WARN_DEPRECATED_OPTC, "SWIG",1, "-c command line option is deprecated. Use -noruntime instead.\n");
+	      }
+	  } else if ((strcmp(argv[i],"-runtime") == 0)) {
+	    Preprocessor_define((String *) "SWIG_RUNTIME_MODE 1", 0);
+	    Swig_mark_arg(i);
+          } else if ((strcmp(argv[i],"-make_default") == 0) || (strcmp(argv[i],"-makedefault") == 0)) {
+	    GenerateDefault = 1;
+	    Swig_mark_arg(i);
+          } else if ((strcmp(argv[i],"-no_default") == 0) || (strcmp(argv[i],"-nodefault") == 0)) {
+	    GenerateDefault = 0;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-noexcept") == 0) {
+	    NoExcept = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-noextern") == 0) {
+	    NoExtern = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-show_templates") == 0) {
+	    Swig_cparse_debug_templates(1);
+	    Swig_mark_arg(i);
+          } else if (strcmp(argv[i],"-swiglib") == 0) {
+	    printf("%s\n", LibDir);
+	    SWIG_exit (EXIT_SUCCESS);
+	  } else if (strcmp(argv[i],"-o") == 0) {
+	      Swig_mark_arg(i);
+	      if (argv[i+1]) {
+		outfile_name = Swig_copy_string(argv[i+1]);
+		Swig_mark_arg(i+1);
+		i++;
+	      } else {
+		Swig_arg_error();
+	      }
+	  } else if (strcmp(argv[i],"-version") == 0) {
+ 	      fprintf(stderr,"\nSWIG Version %s\n", PACKAGE_VERSION);
+	      fprintf(stderr,"Copyright (c) 1995-1998\n");
+	      fprintf(stderr,"University of Utah and the Regents of the University of California\n");
+              fprintf(stderr,"Copyright (c) 1998-2003\n");
+	      fprintf(stderr,"University of Chicago\n");
+	      fprintf(stderr,"Compiled with %s [%s]\n", SWIG_CXX, SWIG_PLATFORM);
+	      fprintf(stderr,"\nPlease see %s for reporting bugs and further information\n", PACKAGE_BUGREPORT);
+	      SWIG_exit (EXIT_SUCCESS);
+	  } else if (strncmp(argv[i],"-l",2) == 0) {
+	    // Add a new directory search path
+	    Append(libfiles,argv[i]+2);
+	    Swig_mark_arg(i);
+          } else if (strcmp(argv[i],"-co") == 0) {
+	    checkout = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-freeze") == 0) {
+	    freeze = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-includeall") == 0) {
+	    Preprocessor_include_all(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-importall") == 0) {
+	    Preprocessor_import_all(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-ignoremissing") == 0) {
+	    Preprocessor_ignore_missing(1);
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-tm_debug") == 0) {
+	    tm_debug = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-module") == 0) {
+	    Swig_mark_arg(i);
+	    if (argv[i+1]) {
+	      ModuleName = NewString(argv[i+1]);	    
+	      Swig_mark_arg(i+1);
+	    } else {
+	      Swig_arg_error();
+	    }
+	  } else if (strcmp(argv[i],"-M") == 0) {
+	    depend = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-MM") == 0) {
+	    depend = 2;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-outdir") == 0) {
+	    Swig_mark_arg(i);
+	    if (argv[i+1]) {
+	      outdir = NewString(argv[i+1]);	    
+	      Swig_mark_arg(i+1);
+	    } else {
+	      Swig_arg_error();
+	    }
+	  } else if (strcmp(argv[i],"-Wall") == 0) {
+	    Swig_mark_arg(i);
+	    Swig_warnall();
+	  } else if (strcmp(argv[i],"-Wallkw") == 0) {
+	    allkw = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-Werror") == 0) {
+	    werror = 1;
+	    Swig_mark_arg(i);
+	  } else if (strncmp(argv[i],"-w",2) == 0) {
+	    Swig_mark_arg(i);
+	    Swig_warnfilter(argv[i]+2,1);
+	  } else if (strcmp(argv[i],"-dump_tags") == 0) {
+	    dump_tags = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_tree") == 0) {
+	    dump_tree = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-nocontract") == 0) {
+	    Swig_mark_arg(i);
+	    Swig_contract_mode_set(0);
+	  } else if (strcmp(argv[i],"-browse") == 0) {
+	    browse = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_typedef") == 0) {
+	    dump_typedef = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_classes") == 0) {
+	    dump_classes = 1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-dump_memory") == 0) {
+	    memory_debug =1;
+	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-help") == 0) {
+	    fputs(usage,stderr);
+	    Swig_mark_arg(i);
+	    help = 1;
+	  }
+      }
+  }
 
-  SWIG_getoptions(argc, argv);
+  for (i = 0; i < includecount; i++) {
+    Swig_add_directory((DOH *) includefiles[i]);
+  }
 
   // Define the __cplusplus symbol
   if (CPlusPlus)
-    Preprocessor_define((DOH *) "__cplusplus __cplusplus", 0);
+    Preprocessor_define((DOH *) "__cplusplus 1", 0);
 
   // Parse language dependent options
   lang->main(argc,argv);
@@ -701,8 +588,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	if (!f) {
 	  fprintf(stderr,"Unable to create file '%s'\n", outfile);
 	} else {
-	  if (Verbose)
-	    fprintf(stdout,"'%s' checked out from the SWIG library.\n", input_file);
+	  fprintf(stderr,"'%s' checked out from the SWIG library.\n", input_file);
 	  fputs(Char(s),f);
 	  fclose(f);
 	}
@@ -726,26 +612,21 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	SWIG_exit (EXIT_FAILURE);
       }
       fclose(df);
-      if(!no_cpp) {
-        Printf(fs,"%%include <swig.swg>\n");
-        if (allkw) {
-          Printf(fs,"%%include <allkw.swg>\n");
-        }
-        if (lang_config) {
-          Printf(fs,"\n%%include <%s>\n", lang_config);
-        }
-        Printf(fs,"%%include \"%s\"\n", Swig_last_file());
-        for (i = 0; i < Len(libfiles); i++) {
-          Printf(fs,"\n%%include \"%s\"\n", Getitem(libfiles,i));
-        }
-        Seek(fs,0,SEEK_SET);
-        cpps = Preprocessor_parse(fs);
-      } else {
-        df = Swig_open(input_file);
-        cpps = NewFileFromFile(df);
+      Printf(fs,"%%include \"swig.swg\"\n");
+      if (allkw) {
+	Printf(fs,"%%include \"allkw.swg\"\n");
       }
+      if (lang_config) {
+	Printf(fs,"\n%%include \"%s\"\n", lang_config);
+      }
+      Printf(fs,"%%include \"%s\"\n", Swig_last_file());
+      for (i = 0; i < Len(libfiles); i++) {
+	Printf(fs,"\n%%include \"%s\"\n", Getitem(libfiles,i));
+      }
+      Seek(fs,0,SEEK_SET);
+      cpps = Preprocessor_parse(fs);
       if (Swig_error_count()) {
-        SWIG_exit(EXIT_FAILURE);
+	SWIG_exit(EXIT_FAILURE);
       }
       if (cpp_only) {
 	Printf(stdout,"%s", cpps);
@@ -763,26 +644,15 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	} else {
 	  outfile = NewString(outfile_name);
 	}
-	if (dependencies_file && Len(dependencies_file) != 0)
-	  f_dependencies_file = NewFile(dependencies_file,"w");
-	else if (!depend_only) {
-	  String *filename = NewStringf("%s_wrap.%s", Swig_file_basename(input_file), depends_extension);
-	  f_dependencies_file = NewFile(filename,"w");
-	} 
-	else
-	  f_dependencies_file = stdout;
-	Printf(f_dependencies_file,"%s: ", outfile);
+	Printf(stdout,"%s: ", outfile);
 	List *files = Preprocessor_depend();
 	for (int i = 0; i < Len(files); i++) {
 	  if ((depend != 2) || ((depend == 2) && (Strncmp(Getitem(files,i),SwigLib, Len(SwigLib)) != 0))) {
-	    Printf(f_dependencies_file,"\\\n %s ", Getitem(files,i));
+	    Printf(stdout,"\\\n %s ", Getitem(files,i));
 	  }
 	}
-	Printf(f_dependencies_file,"\n");
-	if (f_dependencies_file != stdout)
-	  Close(f_dependencies_file);
-	if (depend_only)
-	  SWIG_exit(EXIT_SUCCESS);
+	Printf(stdout,"\n");
+	SWIG_exit(EXIT_SUCCESS);
       }
       Seek(cpps, 0, SEEK_SET);
     }
@@ -845,13 +715,15 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	  } else {
 	    Setattr(top,"outfile", NewStringf("%s_wrap.c", Swig_file_basename(input_file)));
 	  }
+	  Setattr(top,"outfile_h", NewStringf("%s_wrap.h", Swig_file_basename(input_file)));
 	} else {
+	  char *header = strdup(outfile_name);
+	  char *ext = header + strlen(header);
+	  while (ext > header && *ext != '.') ext--;
+	  if (*ext == '.') *ext = 0;
 	  Setattr(top,"outfile", outfile_name);
-	}
-	if (!outfile_name_h) {
-	  Setattr(top,"outfile_h", NewStringf("%s_wrap.%s", Swig_file_basename(input_file),hpp_extension));
-	} else {
-	  Setattr(top,"outfile_h", outfile_name_h);
+	  Setattr(top,"outfile_h", NewStringf("%s.h", header));
+	  free(header);
 	}
         set_outdir(Swig_file_dirname(Getattr(top,"outfile")));
 	if (Swig_contract_mode_get()) {
@@ -866,9 +738,6 @@ int SWIG_main(int argc, char *argv[], Language *l) {
     if (dump_tree) {
       Swig_print_tree(top);
     }
-    if (dump_xml) {
-      Swig_print_xml(top, xmlout);
-    }
   }
   if (tm_debug) Swig_typemap_debug();
   if (memory_debug) DohMemoryDebug();
