@@ -8,10 +8,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.53 2004/11/15 22:52:18 marcelomatus Exp $";
+char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.39 2003/12/09 02:52:07 mmatus Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx,v 1.53 2004/11/15 22:52:18 marcelomatus Exp $
+ * $Header: /cvsroot/SWIG/Source/Modules/ocaml.cxx,v 1.39 2003/12/09 02:52:07 mmatus Exp $
  *
  * ocaml.cxx
  *
@@ -20,13 +20,16 @@ char cvsroot_ocaml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ocaml.cxx
 
 #include "swigmod.h"
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 #include <ctype.h>
 
 static const char *usage = (char*)
     ("Ocaml Options (available with -ocaml)\n"
      "-prefix <name>  - Set a prefix <name> to be prepended to all names\n"
      "-where          - Emit library location\n"
-     "-suffix <name>  - Change .cxx to something else\n"
      "\n");
 
 static int classmode = 0;
@@ -56,8 +59,6 @@ static  File         *f_mlout = 0;
 static  File         *f_mliout = 0;
 static  File         *f_mlbody = 0;
 static  File         *f_mlibody = 0;
-static  File         *f_mltail = 0;
-static  File         *f_mlitail = 0;
 static  File         *f_enumtypes_type = 0;
 static  File         *f_enumtypes_value = 0;
 static  File         *f_class_ctors = 0;
@@ -68,20 +69,10 @@ static  File         *f_int_to_enum = 0;
 class OCAML : public Language {
 public:
 
-  OCAML() 
-  {
-    director_prot_ctor_code = NewString("");    
-    Printv(director_prot_ctor_code,
-	   "if ( $comparison ) { /* subclassed */\n",
-	   "  $director_new \n",
-	   "} else {\n",
-	   "  failwith(\"accessing abstract class or protected constructor\"); \n",
-	   "}\n", NIL);
-    director_multiple_inheritance = 1;
-    director_language = 1;
-  }
-  
-      
+    int validIdentifier( String *s ) {
+	return true;
+    }
+
     String *Swig_class_name(Node *n) {
 	String *name;
 	name = Copy(Getattr(n, "sym:name"));
@@ -123,14 +114,6 @@ public:
 		    } else {
 			Swig_arg_error();
 		    }
-		} else if (strcmp (argv[i], "-suffix") == 0) {
-		    if (argv[i + 1]) {
-			SWIG_config_cppext( argv[i+1] );
-			Swig_mark_arg (i);
-			Swig_mark_arg (i+1);
-			i++;
-		    } else
-			Swig_arg_error();
 		}
 	    }
 	}
@@ -241,8 +224,6 @@ public:
 	init_func_def = NewString("");
 	f_mlbody = NewString("");
 	f_mlibody = NewString("");
-	f_mltail = NewString("");
-	f_mlitail = NewString("");
 	f_class_ctors = NewString("");
 	f_class_ctors_end = NewString("");
 	f_enum_to_int = NewString("");
@@ -262,8 +243,6 @@ public:
 	Swig_register_filebyname("runtime",f_runtime);
 	Swig_register_filebyname("mli",f_mlibody);
 	Swig_register_filebyname("ml",f_mlbody);
-	Swig_register_filebyname("mlitail",f_mlitail);
-	Swig_register_filebyname("mltail",f_mltail);
 	Swig_register_filebyname("director",f_directors);
 	Swig_register_filebyname("director_h",f_directors_h);
 	Swig_register_filebyname("classtemplate",f_classtemplate);
@@ -279,19 +258,18 @@ public:
 	Printf( f_mlbody, "let module_name = \"%s\"\n", module );
 	Printf( f_mlibody, "val module_name : string\n" );
 	Printf( f_enum_to_int, 
-		"let enum_to_int x (v : c_obj) =\n"
-		"   match v with\n"
-		"     C_enum _y ->\n"
-		"     (let y = _y in match (x : c_enum_type) with\n"
-		"       `unknown -> "
-		"         (match y with\n"
-		"           `Int x -> (Swig.C_int x)\n"
-		"           | _ -> raise (LabelNotFromThisEnum v))\n" );
+		"let enum_to_int x v =\n"
+		"  match v with C_enum y -> (\n"
+		"  match (x : c_enum_type) with\n"
+		"   `unknown -> "
+		"    (match (y : c_enum_tag) with\n"
+		"      `int (x : int) -> C_int x\n"
+		"    | _ -> raise (LabelNotFromThisEnum v))\n" );
 
 	Printf( f_int_to_enum,
 		"let int_to_enum x y =\n"
 		"    match (x : c_enum_type) with\n"
-		"      `unknown -> C_enum (`Int y)\n" );
+		"      `unknown -> C_enum (`int y)\n" );
 
 	Swig_banner (f_runtime);
 
@@ -300,11 +278,18 @@ public:
 	    Swig_insert_file("director.swg", f_directors_h);
 	}
     
+	if (NoInclude) {
+	    Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
+	}
+
 	/* Produce the enum_to_int and int_to_enum functions */
     
-	Printf(f_enumtypes_type,"open Swig\n"
-	       "type c_enum_type = [ \n  `unknown\n" );
-	Printf(f_enumtypes_value,"type c_enum_value = [ \n  `Int of int\n" );
+	Printf(f_enumtypes_type,"type c_enum_type = [ \n  `unknown\n" );
+	Printf(f_enumtypes_value,
+	       "type c_enum_tag = [\n"
+	       "  `SWIGFake\n"
+	       "| `int of int\n" );
+    
 	String *mlfile = NewString("");
 	String *mlifile = NewString("");
 
@@ -329,7 +314,7 @@ public:
 		"let _ = Callback.register \"%s_enum_to_int\" enum_to_int\n", 
 		module );
 	Printf( f_mlibody, 
-		"val enum_to_int : c_enum_type -> c_obj -> Swig.c_obj\n" );
+		"val enum_to_int : c_enum_type -> c_obj -> c_obj\n" );
 
 	Printf( f_int_to_enum,
 		"let _ = Callback.register \"%s_int_to_enum\" int_to_enum\n",
@@ -346,8 +331,7 @@ public:
 		"let _ = f_init ()\n",
 		module, module );
 	Printf( f_enumtypes_type, "]\n" );
-	Printf( f_enumtypes_value, "]\n\n"
-		"type c_obj = c_enum_value c_obj_t\n" );
+	Printf( f_enumtypes_value, "]\n" );
 
 	SwigType_emit_type_table (f_runtime, f_wrappers);
 	/* Close all of the files */
@@ -371,14 +355,12 @@ public:
 	Delete(f_enum_to_int);
 	Dump(f_class_ctors,f_mlout);
 	Dump(f_class_ctors_end,f_mlout);
-	Dump(f_mltail,f_mlout);
 	Close(f_mlout);
 	Delete(f_mlout);
 
 	Dump(f_enumtypes_type,f_mliout);
 	Dump(f_enumtypes_value,f_mliout);
 	Dump(f_mlibody,f_mliout);
-	Dump(f_mlitail,f_mliout);
 	Close(f_mliout);
 	Delete(f_mliout);
 
@@ -468,7 +450,6 @@ public:
 	int newobj = Getattr(n,"feature:new") ? 1 : 0;
 	String *nodeType = Getattr(n, "nodeType");
 	int constructor = !Cmp(nodeType, "constructor");
-	int destructor = (!Cmp(nodeType, "destructor")); 
 	String *storage = Getattr(n,"storage");
 	int isVirtual = !Cmp(storage,"virtual");
 	String *overname = 0;
@@ -484,11 +465,8 @@ public:
 	if (overname) {
 	    Append(wname, overname);
 	}
-	/* Do this to disambiguate functions emitted from different modules */
-	Append(wname, module);
-
 	Setattr(n,"wrap:name",wname);
-
+    
 	// Build the name for Scheme.
 	Printv(proc_name,"_",iname,NIL);
 	String *mangled_name = mangleNameForCaml(proc_name);
@@ -689,10 +667,9 @@ public:
 	// (the smart-pointer) and the director object (the "pointee") are
 	// distinct.
 
-	if (directorsEnabled()) {
-	  if (!is_smart_pointer()) {
-	    if (/*directorbase &&*/ !constructor && !destructor 
-		&& isVirtual  && !Getattr(n,"feature:nodirector")) {
+	if (CPlusPlus && directorsEnabled()) {
+	    if (!is_smart_pointer()) {
+		if (/*directorbase &&*/ !constructor && isVirtual) {
 		    Wrapper_add_local(f, "director", "Swig::Director *director = 0");
 		    Printf(f->code, "director = dynamic_cast<Swig::Director *>(arg1);\n");
 			   
@@ -757,6 +734,7 @@ public:
 	    if( !Getattr(n,"sym:nextSibling") ) {
 		int maxargs;
 		Wrapper *df = NewWrapper();
+		String *dname = Swig_name_wrapper(iname);
 		String *dispatch = 
 		    Swig_overload_dispatch(n,
 					   "free(argv);\n"
@@ -765,15 +743,9 @@ public:
 
 		Wrapper_add_local(df, "_v", "int _v = 0");
 		Wrapper_add_local(df, "argv", "CAML_VALUE *argv");
-		
-		/* Undifferentiate name .. this is the dispatch function */
-		wname = Swig_name_wrapper(iname);
-		/* Do this to disambiguate functions emitted from different
-		 * modules */
-		Append(wname, module);
 
 		Printv(df->def,
-		       "SWIGEXT CAML_VALUE ",wname,"(CAML_VALUE args) {\n"
+		       "SWIGEXT CAML_VALUE ",dname,"(CAML_VALUE args) {\n"
 		       "  CAMLparam1(args);\n"
 		       "  int i;\n"
 		       "  int argc = caml_list_length(args);\n",NIL);
@@ -787,25 +759,34 @@ public:
 		Printv(df->code,"}\n",NIL);
 		Wrapper_print(df,f_wrappers);
 
+		Printf(f_mlbody, 
+		       "external %s_f : c_obj list -> c_obj list = \"%s\" ;;\n"
+		       "let %s = fnhelper %s %s_f\n",
+		       mangled_name, dname, mangled_name, 
+		       newobj ? "true" : "false",
+		       mangled_name );
+		if( !classmode || in_constructor || in_destructor ||
+		    static_member_function ) 
+		    Printf(f_mlibody,
+			   "(* overload *)\n"
+			   "val %s : c_obj -> c_obj\n", mangled_name );
+
 		DelWrapper(df);
 		Delete(dispatch);
+		Delete(dname);
 	    }
+	} else {
+	    Printf(f_mlbody, 
+		   "external %s_f : c_obj list -> c_obj list = \"%s\" ;;\n"
+		   "let %s = fnhelper %s %s_f\n",
+		   mangled_name, wname, mangled_name, newobj ? "true" : "false", 
+		   mangled_name );
+	    if( !classmode || in_constructor || in_destructor ||
+		static_member_function ) 
+		Printf(f_mlibody,
+		       "(* Non-overload *)\n"
+		       "val %s : c_obj -> c_obj\n", mangled_name );
 	}
-
-	Printf(f_mlbody, 
-	       "external %s_f : c_obj list -> c_obj list = \"%s\" ;;\n"
-	       "let %s arg = match %s_f (fnhelper arg) with\n"
-	       "  [] -> C_void\n"
-	       "| [x] -> (if %s then Gc.finalise \n"
-	       "  (fun x -> ignore ((invoke x) \"~\" C_void)) x) ; x\n"
-	       "| lst -> C_list lst ;;\n",
-	       mangled_name, wname, 
-	       mangled_name, mangled_name, newobj ? "true" : "false");
-	
-	if( !classmode || in_constructor || in_destructor ||
-	    static_member_function ) 
-	    Printf(f_mlibody,
-		   "val %s : c_obj -> c_obj\n", mangled_name );
    
 	Delete(proc_name);
 	Delete(source);
@@ -918,15 +899,16 @@ public:
 	
 	if( Getattr( n, "feature:immutable" ) ) {
 	    Printf( f_mlbody, 
-		    "external _%s : c_obj -> Swig.c_obj = \"%s\" \n",
-		    mname, var_name );
-	    Printf( f_mlibody, "val _%s : c_obj -> Swig.c_obj\n", iname );
+		    "external __%s : c_obj -> c_obj = \"%s\" \n"
+		    "let _%s = __%s C_void\n",
+		    mname, var_name, mname, mname );
+	    Printf( f_mlibody, "val _%s : c_obj\n", iname );
 	    if( const_enum ) {
 		Printf( f_enum_to_int, 
-			" | `%s -> _%s C_void\n", 
+			" | `%s -> _%s\n", 
 			mname, mname );
 		Printf( f_int_to_enum, 
-			" if y = (get_int (_%s C_void)) then `%s else\n",
+			" if y = (get_int _%s) then `%s else\n",
 			mname, mname );
 	    }
 	} else {
@@ -1093,12 +1075,6 @@ public:
 	      strchr(ch,')') || strchr(ch,'>'));
     }
 
-    /* We accept all chars in identifiers because we use strings to index
-     * them. */
-    int validIdentifier( String *name ) {
-	return Len(name) > 0 ? 1 : 0; 
-    }
-
     /* classHandler
      * 
      * Create a "class" definition for ocaml.  I thought quite a bit about
@@ -1215,14 +1191,13 @@ public:
 	  Iterator b;
 	  b = First(baselist);
 	  while (b.item) {
-	    String *bname = Getattr(b.item, "name");
+	    String *bname = Getattr(b.item, "ocaml:ctor");
 	    if (bname) {
-	      String *base_create = NewString("");
-	      Printv(base_create,"(create_class \"",bname,"\")",NIL);
 	      Printv(f_class_ctors,
-		     "   \"::",bname,"\", (fun args -> ",
-		     base_create," args) ;\n",NIL);
-	      Printv( base_classes, base_create, " ;\n", NIL );
+		     "   \"::",bname,"\", (fun args -> "
+		     "create_",bname,"_from_ptr raw_ptr) ;\n",NIL);
+	      Printv( base_classes, "create_", bname, "_from_ptr ;\n", 
+		      NIL );
 	    }
 	    b = Next(b);
 	  }
@@ -1382,38 +1357,19 @@ public:
      * which means looking up and registering by typedef and enum name. */
     int enumDeclaration(Node *n) {
 	String *name = Getattr(n,"name");
-	String *oname = name ? NewString(name) : NULL;
-	/* name is now fully qualified */
-	String *fully_qualified_name = NewString(name);
+	String *fully_qualified_name = fully_qualify_enum_name(n,name);
 	bool seen_enum = false;
 	if( name_qualifier ) 
 	    Delete(name_qualifier);
-	char *strip_position;
 	name_qualifier = fully_qualify_enum_name(n,NewString(""));
 	
-	/* Recent changes have distrubed enum and template naming again.
-	 * Will try to keep it consistent by can't guarantee much given
-	 * that these things move around a lot.
-	 *
-	 * I need to figure out a way to isolate this module better.
-	 */
-	if( oname ) {
-	    strip_position = strstr(Char(oname),"::");
-	    
-	    while( strip_position ) {
-		strip_position += 2;
-		oname = NewString( strip_position );
-		strip_position = strstr( Char(oname), "::" );
-	    }
-	}
-
-	seen_enum = oname ? 
+	seen_enum = name ? 
 	    (Getattr(seen_enums,fully_qualified_name) ? true : false) : false;
 
-	if( oname && !seen_enum ) {
+	if( name && !seen_enum ) {
 	    const_enum = true;
-	    Printf( f_enum_to_int, "| `%s -> (match y with\n", oname );
-	    Printf( f_int_to_enum, "| `%s -> C_enum (\n", oname );
+	    Printf( f_enum_to_int, "| `%s -> (match (y : c_enum_tag) with\n", name );
+	    Printf( f_int_to_enum, "| `%s -> C_enum (\n", name );
 	    /* * * * A note about enum name resolution * * * *
 	     * This code should now work, but I think we can do a bit better.
 	     * The problem I'm having is that swig isn't very precise about
@@ -1425,17 +1381,15 @@ public:
 	     * * * */
 	    Printf( f_mlbody, 
 		    "let _ = Callback.register \"%s_marker\" (`%s)\n",
-		    fully_qualified_name, oname );
+		    fully_qualified_name, name );
 	    if( !strncmp(Char(fully_qualified_name),"enum ",5) ) {
 		String *fq_noenum = NewString(Char(fully_qualified_name) + 5);
 		Printf( f_mlbody,
-			"let _ = Callback.register \"%s_marker\" (`%s)\n"
 			"let _ = Callback.register \"%s_marker\" (`%s)\n",
-			fq_noenum, oname,
 			fq_noenum, name );
 	    }
 
-	    Printf( f_enumtypes_type,"| `%s\n", oname );
+	    Printf( f_enumtypes_type,"| `%s\n", name );
 	    Insert(fully_qualified_name,0,"enum ");
 	    Setattr(seen_enums,fully_qualified_name,n);
 	}
@@ -1443,10 +1397,10 @@ public:
 	int ret = Language::enumDeclaration(n);
 	
 	if( const_enum ) {
-	    Printf( f_int_to_enum, "`Int y)\n" );
+	    Printf( f_int_to_enum, "`int y)\n", name );
 	    Printf( f_enum_to_int, 
-		    "| `Int x -> Swig.C_int x\n"
-		    "| _ -> raise (LabelNotFromThisEnum v))\n" );
+		    "| `int (x : int) -> C_int x\n"
+		    "| _ -> raise (Failure \"Unknown enum tag\"))\n" );
 	}
 
 	const_enum = false;
@@ -1757,18 +1711,8 @@ public:
 
 	/* any existing helper functions to handle this? */
 	if (!is_void) {
-	    /* A little explanation:
-	     * The director_enum test case makes a method whose return type
-	     * is an enum type.  return_type here is "int".  gcc complains
-	     * about an implicit enum conversion, and although i don't strictly
-	     * agree with it, I'm working on fixing the error:
-	     *
-	     * Below is what I came up with.  It's not great but it should
-	     * always essentially work.
-	     */
-	    if( !SwigType_isreference(return_type) ) {
-		Printf(w->code, "CAMLreturn((%s)c_result);\n",
-		       SwigType_lstr(return_type, ""));
+	    if (!SwigType_isreference(return_type)) {
+		Printf(w->code, "CAMLreturn(c_result);\n");
 	    } else {
 		Printf(w->code, "CAMLreturn(*c_result);\n");
 	    }
@@ -1778,10 +1722,8 @@ public:
 
 	/* emit the director method */
 	if (status == SWIG_OK) {
-          if (!Getattr(n,"defaultargs")) {
 	    Wrapper_print(w, f_directors);
 	    Printv(f_directors_h, declaration, NIL);
-          }
 	}
 
 	/* clean up */
@@ -1808,8 +1750,8 @@ public:
 	String *classname = NewString("");
 	Printf(classname, "SwigDirector_%s", supername);
 
-	/* insert self parameter */
-	Parm *p, *q;
+	/* insert self and disown parameters */
+	Parm *p, *q, *ip;
 	ParmList *superparms = Getattr(n, "parms");
 	ParmList *parms = CopyParmList(superparms);
 	String *type = NewString("CAML_VALUE");
@@ -1818,29 +1760,37 @@ public:
 	set_nextSibling(q, superparms);
 	set_nextSibling(p, parms);
 	parms = p;
+	for (ip = parms; nextSibling(ip); ) ip = nextSibling(ip);
+	p = NewParm(NewString("bool"), NewString("disown"));
+	Setattr(p, "arg:byname", "1");
+	Setattr(n, "director:postfix_args", p);
+	Setattr(p, "value", "0");
+	set_nextSibling(ip, p);
 
-        if (!Getattr(n,"defaultargs")) {
-          /* constructor */
-          {
-              Wrapper *w = NewWrapper();
-              String *call;
-              String *basetype = Getattr(parent, "classtype");
-              String *target = Swig_method_decl(decl, classname, parms, 0, 0);
-              call = Swig_csuperclass_call(0, basetype, superparms);
-              Printf( w->def, "%s::%s: %s, Swig::Director(self) { }", classname, target, call );
-              Delete(target);
-              Wrapper_print(w, f_directors);
-              Delete(call);
-              DelWrapper(w);
-          }
-      
-          /* constructor header */
-          {
-              String *target = Swig_method_decl(decl, classname, parms, 0, 1);
-              Printf(f_directors_h, "    %s;\n", target);
-              Delete(target);
-          }
-        }
+	/* constructor */
+	{
+	    Wrapper *w = NewWrapper();
+	    String *call;
+	    String *basetype = Getattr(parent, "classtype");
+	    String *target = Swig_method_decl(decl, classname, parms, 
+					 0, 0);
+	    call = Swig_csuperclass_call(0, basetype, superparms);
+	    Printf( w->def, 
+		    "%s::%s: %s, Swig::Director(self, disown) { }", 
+		    classname, target, call );
+	    Delete(target);
+	    Wrapper_print(w, f_directors);
+	    Delete(call);
+	    DelWrapper(w);
+	}
+    
+	/* constructor header */
+	{
+	    String *target = Swig_method_decl(decl, classname, 
+					 parms, 0, 1);
+	    Printf(f_directors_h, "    %s;\n", target);
+	    Delete(target);
+	}
 
 	Setattr(n,"parms",q);
 	Language::classDirectorConstructor(n);
@@ -1861,8 +1811,8 @@ public:
 	String *classname;
 	classname = Swig_class_name(n);
 
-	/* insert self parameter */
-	Parm *p, *q;
+	/* insert self and disown parameters */
+	Parm *p, *q, *ip;
 	ParmList *superparms = Getattr(n, "parms");
 	ParmList *parms = CopyParmList(superparms);
 	String *type = NewString("CAML_VALUE");
@@ -1870,14 +1820,20 @@ public:
 	q = Copy(p);
 	set_nextSibling(p, parms);
 	parms = p;
+	for (ip = parms; nextSibling(ip); ) ip = nextSibling(ip);
+	p = NewParm(NewString("bool"), NewString("disown"));
+	Setattr(p, "arg:byname", "1");
+	Setattr(n, "director:postfix_args", p);
+	Setattr(p, "value", "0");
+	set_nextSibling(ip, p);
 
 	{
 	    Wrapper *w = NewWrapper();
-	    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(CAML_VALUE self) : Swig::Director(self) { }", classname, classname);
+	    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(CAML_VALUE self, bool disown) : Swig::Director(self, disown) { }", classname, classname);
 	    Wrapper_print(w, f_directors);
 	    DelWrapper(w);
 	}
-	Printf(f_directors_h, "    SwigDirector_%s(CAML_VALUE self);\n", classname);
+	Printf(f_directors_h, "    SwigDirector_%s(CAML_VALUE self, bool disown = true);\n", classname);
 	Delete(classname);
 	Setattr(n,"parms",q);
 	return Language::classDirectorDefaultConstructor(n);
@@ -1937,10 +1893,8 @@ public:
  * swig_ocaml()    - Instantiate module
  * ------------------------------------------------------------------------- */
 
-static Language * new_swig_ocaml() {
+extern "C" Language *
+swig_ocaml(void) {
     return new OCAML();
 }
-extern "C" Language * swig_ocaml(void) {
-    return new_swig_ocaml();
-}
 
