@@ -13,14 +13,17 @@
  * See the file LICENSE for information on usage and redistribution.
  * ------------------------------------------------------------------------- */
 
-char cvsroot_perl5_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/perl5.cxx,v 1.17 2004/12/01 00:50:05 marcelomatus Exp $";
+char cvsroot_perl5_cxx[] = "$Header: /cvsroot/SWIG/Source/Modules/perl5.cxx,v 1.9 2003/12/17 19:09:36 beazley Exp $";
 
 #include "swigmod.h"
 
-#include <ctype.h>
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
 
 static const char *usage = (char*)"\
 Perl5 Options (available with -perl5)\n\
+     -ldflags        - Print runtime libraries to link with\n\
      -static         - Omit code related to dynamic loading\n\
      -nopm           - Do not generate the .pm file\n\
      -proxy          - Create proxy classes\n\
@@ -96,7 +99,6 @@ static  int       num_consts = 0;            /* Number of constants */
 static  String   *var_stubs = 0;             /* Variable stubs */
 static  String   *exported = 0;              /* Exported symbols */
 static  String   *pragma_include = 0;
-static  String   *additional_perl_code = 0;  /* Additional Perl code from %perlcode %{ ... %} */
 static  Hash     *operators = 0;
 static  int       have_operators = 0;
 
@@ -162,6 +164,9 @@ public:
 	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-help") == 0) {
 	  fputs(usage,stderr);
+	} else if (strcmp (argv[i], "-ldflags") == 0) {
+	  printf("%s\n", SWIG_PERL_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
 	}
       }
     }
@@ -206,7 +211,6 @@ public:
     exported       = NewString("");
     magic          = NewString("");
     pragma_include = NewString("");
-    additional_perl_code = NewString("");
 
     command_tab    = NewString("static swig_command_info swig_commands[] = {\n");
     constant_tab   = NewString("static swig_constant_info swig_constants[] = {\n");
@@ -214,6 +218,10 @@ public:
 
     Swig_banner(f_runtime);
   
+    if (NoInclude) {
+      Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
+    }
+
     module = Copy(Getattr(n,"name"));
 
   /* If we're in blessed mode, change the package name to "packagec" */
@@ -250,7 +258,6 @@ public:
       }
       Delete(filen); filen = NULL;
       Swig_register_filebyname("pm",f_pm);
-      Swig_register_filebyname("perl",f_pm);
     }
     {
       String *tmp = NewString(fullmodule);
@@ -333,7 +340,7 @@ public:
     Printf(f_wrappers,"%s",type_table);
     Delete(type_table);
 
-    Printf(constant_tab,"{0,0,0,0,0,0}\n};\n");
+    Printf(constant_tab,"{0}\n};\n");
     Printv(f_wrappers,constant_tab,NIL);
 
     Printf(f_wrappers,"#ifdef __cplusplus\n}\n#endif\n");
@@ -343,7 +350,7 @@ public:
     Printf(f_init,"}\n");
 
     /* Finish off tables */
-    Printf(variable_tab, "{0,0,0,0}\n};\n");
+    Printf(variable_tab, "{0}\n};\n");
     Printv(f_wrappers,variable_tab,NIL);
 
     Printf(command_tab,"{0,0}\n};\n");
@@ -360,9 +367,9 @@ public:
       Printf(f_pm,"boot_%s();\n", tmp);
       Delete(tmp);
     }
+    Printf(f_pm,"%s",pragma_include);
     Printf(f_pm,"package %s;\n", fullmodule);
     Printf(f_pm,"@EXPORT = qw( %s);\n",exported);
-    Printf(f_pm,"%s",pragma_include);
 
     if (blessed) {
 
@@ -446,9 +453,6 @@ public:
       Printf(f_pm,"%s",var_stubs);
     }
 
-    /* Add additional Perl code at the end */
-    Printf(f_pm,"%s",additional_perl_code);
-
     Printf(f_pm,"1;\n");
     Close(f_pm);
     Delete(f_pm);
@@ -1071,13 +1075,10 @@ public:
    * ------------------------------------------------------------ */
   virtual int classDeclaration(Node *n) {
     /* Do some work on the class name */
-    if (!Getattr(n,"feature:onlychildren")) {
-      if (blessed) {
-	setclassname(n);
-	Append(classlist,n);
-      }
+    if (blessed) {
+      setclassname(n);
+      Append(classlist,n);
     }
-    
     return Language::classDeclaration(n);
   }
   
@@ -1231,15 +1232,7 @@ public:
 	//      fprintf(stderr,"Found member_func operator: %s\n", symname);
       }
 
-      if (Getattr(n,"feature:shadow")) {
-	String *plcode = perlcode(Getattr(n,"feature:shadow"),0);
-	String *plaction = NewStringf("%s::%s", module, Swig_name_member(class_name,symname));
-	Replaceall(plcode,"$action" ,plaction);
-	Delete(plaction);
-	Printv(pcode,plcode,NIL);
-      } else {
-	Printv(pcode,"*",symname," = *", cmodule, "::", Swig_name_member(class_name,symname), ";\n", NIL);
-      }
+      Printv(pcode,"*",symname," = *", cmodule, "::", Swig_name_member(class_name,symname), ";\n", NIL);
     }
     return SWIG_OK;
   }
@@ -1300,30 +1293,22 @@ public:
     Language::constructorHandler(n);
 
     if ((blessed) && (!Getattr(n,"sym:nextSibling"))) {
-      if (Getattr(n,"feature:shadow")) {
-        String *plcode = perlcode(Getattr(n,"feature:shadow"),0);
-        String *plaction = NewStringf("%s::%s", module, Swig_name_member(class_name,symname));
-	Replaceall(plcode,"$action" ,plaction);
-        Delete(plaction);
-        Printv(pcode,plcode,NIL);
+      if ((Cmp(symname,class_name) == 0)) {
+	/* Emit a blessed constructor  */
+	Printf(pcode, "sub new {\n");
       } else {
-	if ((Cmp(symname,class_name) == 0)) {
-	  /* Emit a blessed constructor  */
-	  Printf(pcode, "sub new {\n");
-	} else {
-	  /* Constructor doesn't match classname so we'll just use the normal name  */
-	  Printv(pcode, "sub ", Swig_name_construct(symname), " () {\n", NIL);
-	}
-
-	Printv(pcode,
-	       tab4, "my $pkg = shift;\n",
-	       tab4, "my $self = ", cmodule, "::", Swig_name_construct(symname), "(@_);\n",
-	       tab4, "bless $self, $pkg if defined($self);\n",
-	       "}\n\n",
-	       NIL);
-
-	have_constructor = 1;
+	/* Constructor doesn't match classname so we'll just use the normal name  */
+	Printv(pcode, "sub ", Swig_name_construct(symname), " () {\n", NIL);
       }
+
+      Printv(pcode,
+	     tab4, "my $pkg = shift;\n",
+	     tab4, "my $self = ", cmodule, "::", Swig_name_construct(symname), "(@_);\n",
+	     tab4, "bless $self, $pkg if defined($self);\n",
+	     "}\n\n",
+	     NIL);
+
+      have_constructor = 1;
     }
     member_func = 0;
     return SWIG_OK;
@@ -1338,26 +1323,18 @@ public:
     member_func = 1;
     Language::destructorHandler(n);
     if (blessed) {
-      if (Getattr(n,"feature:shadow")) {
-        String *plcode = perlcode(Getattr(n,"feature:shadow"),0);
-        String *plaction = NewStringf("%s::%s", module, Swig_name_member(class_name,symname));
-	Replaceall(plcode,"$action" ,plaction);
-        Delete(plaction);
-        Printv(pcode,plcode,NIL);
-      } else {
-	Printv(pcode,
-	       "sub DESTROY {\n",
-	       tab4, "return unless $_[0]->isa('HASH');\n",
-	       tab4, "my $self = tied(%{$_[0]});\n",
-	       tab4, "return unless defined $self;\n",
-	       tab4, "delete $ITERATORS{$self};\n",
-	       tab4, "if (exists $OWNER{$self}) {\n",
-	       tab8,  cmodule, "::", Swig_name_destroy(symname), "($self);\n",
-	       tab8, "delete $OWNER{$self};\n",
-	       tab4, "}\n}\n\n",
-	       NIL);
-	have_destructor = 1;
-      }
+      Printv(pcode,
+	     "sub DESTROY {\n",
+	     tab4, "return unless $_[0]->isa('HASH');\n",
+	     tab4, "my $self = tied(%{$_[0]});\n",
+	     tab4, "return unless defined $self;\n",
+	     tab4, "delete $ITERATORS{$self};\n",
+	     tab4, "if (exists $OWNER{$self}) {\n",
+	     tab8,  cmodule, "::", Swig_name_destroy(symname), "($self);\n",
+	     tab8, "delete $OWNER{$self};\n",
+	     tab4, "}\n}\n\n",
+	     NIL);
+      have_destructor = 1;
     }
     member_func = 0;
     return SWIG_OK;
@@ -1453,89 +1430,13 @@ public:
     }
     return Language::pragmaDirective(n);
   }
-
-  /* ------------------------------------------------------------
-   * perlcode()     - Output perlcode code into the shadow file
-   * ------------------------------------------------------------ */
-
-  String *perlcode(String *code, const String *indent) {
-    String *out = NewString("");
-    String *temp;
-    char   *t;
-    if (!indent) indent = "";
-
-    temp = NewString(code);
-
-    t = Char(temp);
-    if (*t == '{') {
-      Delitem(temp,0);
-      Delitem(temp,DOH_END);
-    }
-
-    /* Split the input text into lines */
-    List *clist = DohSplitLines(temp);
-    Delete(temp);
-    int   initial = 0;
-    String *s = 0;
-    Iterator si;
-    /* Get the initial indentation */
-    
-    for (si = First(clist); si.item; si = Next(si)) {
-      s = si.item;
-      if (Len(s)) {
-	char *c = Char(s);
-	while (*c) {
-	  if (!isspace(*c)) break;
-	  initial++;
-	  c++;
-	}
-	if (*c && !isspace(*c)) break;
-	else {
-	  initial = 0;
-	}
-      }
-    }
-    while (si.item) {
-      s = si.item;
-      if (Len(s) > initial) {
-	char *c = Char(s);
-	c += initial;
-	Printv(out,indent,c,"\n",NIL);
-      } else {
-	Printv(out,"\n",NIL);
-      }
-      si = Next(si);
-    }
-    Delete(clist);
-    return out;
-  }
-  
-  /* ------------------------------------------------------------
-   * insertDirective()
-   * 
-   * Hook for %insert directive.
-   * ------------------------------------------------------------ */
-
-  virtual int insertDirective(Node *n) {
-    String *code = Getattr(n,"code");
-    String *section = Getattr(n,"section");
-
-    if ((!ImportMode) && (Cmp(section,"perl") == 0)) {
-	Printv(additional_perl_code, code, NIL);
-    } else {
-      Language::insertDirective(n);
-    }
-    return SWIG_OK;
-  }
 };
   
 /* -----------------------------------------------------------------------------
  * swig_perl5()    - Instantiate module
  * ----------------------------------------------------------------------------- */
 
-static Language * new_swig_perl5() {
+extern "C" Language *
+swig_perl5(void) {
   return new PERL5();
 }
-extern "C" Language * swig_perl5(void) {
-  return new_swig_perl5();
-}
