@@ -14,7 +14,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/parms.c,v 1.4.4.2 2001/09/02 16:15:48 beazley Exp $";
+char cvsroot_parms_c[] = "$Header: /cvsroot/SWIG/Source/Swig/parms.c,v 1.7 2003/09/04 23:26:29 cheetah Exp $";
 
 #include "swig.h"
 
@@ -42,27 +42,40 @@ Parm *NewParm(SwigType *type, String_or_char *n) {
 
 Parm *CopyParm(Parm *p) {
   SwigType *t;
-  char     *name;
-  char     *lname;
-  char     *value;
-  int       ignore;
+  String   *name;
+  String   *lname;
+  String   *value;
+  String   *ignore;
+  String   *alttype;
+  String   *byname;
 
   Parm *np = NewHash();
   t = Getattr(p,"type");
-  name = GetChar(p,"name");
-  lname = GetChar(p,"lname");
-  value = GetChar(p,"value");
-  ignore = GetInt(p,"ignore");
+  name = Getattr(p,"name");
+  lname = Getattr(p,"lname");
+  value = Getattr(p,"value");
+  ignore = Getattr(p,"ignore");
+  alttype = Getattr(p,"alttype");
+  byname = Getattr(p, "arg:byname");
 
-  Setattr(np,"type",Copy(t));
+  if (t) 
+    Setattr(np,"type",Copy(t));
   if (name)
-    Setattr(np,"name",name);
+    Setattr(np,"name",Copy(name));
   if (lname)
-    Setattr(np,"lname", lname);
+    Setattr(np,"lname", Copy(lname));
   if (value)
-    Setattr(np,"value", value);
+    Setattr(np,"value", Copy(value));
   if (ignore)
-    SetInt(np,"ignore", ignore);
+    Setattr(np,"ignore", Copy(ignore));
+  if (alttype) 
+    Setattr(np,"alttype", Copy(alttype));
+  if (byname)
+    Setattr(np, "arg:byname", Copy(byname));
+      
+  Setfile(np,Getfile(p));
+  Setline(np,Getline(p));
+
   return np;
 }
 
@@ -98,7 +111,7 @@ CopyParmList(ParmList *p) {
 int ParmList_numarg(ParmList *p) {
   int  n = 0;
   while (p) {
-    if (!Getignore(p)) n++;
+    if (!Getattr(p,"ignore")) n++;
     p = nextSibling(p);
   }
   return n;
@@ -111,10 +124,10 @@ int ParmList_numarg(ParmList *p) {
 int ParmList_numrequired(ParmList *p) {
   int i = 0;
   while (p) {
-    SwigType *t = Gettype(p);
-    String   *value = Getvalue(p);
+    SwigType *t = Getattr(p,"type");
+    String   *value = Getattr(p,"value");
     if (value) return i;
-    if (!SwigType_cmp(t,"void")) i++;
+    if (!(SwigType_type(t) == T_VOID)) i++;
     else break;
     p = nextSibling(p);
   }
@@ -146,13 +159,13 @@ String *ParmList_str(ParmList *p) {
 
   out = NewString("");
   while(p) {
-    t = Gettype(p);
-    Printf(out,"%s", SwigType_str(t,Getname(p)));
+    t = Getattr(p,"type");
+    Printf(out,"%s", SwigType_str(t,Getattr(p,"name")));
     p = nextSibling(p);
     if (p)
       Printf(out,",");
   }
-  return Swig_temp_result(out);
+  return out;
 }
 
 /* ---------------------------------------------------------------------
@@ -167,13 +180,17 @@ String *ParmList_protostr(ParmList *p) {
 
   out = NewString("");
   while(p) {
-    t = Gettype(p);
+    if (Getattr(p,"hidden")) {
+      p = nextSibling(p);
+      continue;
+    }
+    t = Getattr(p,"type");
     Printf(out,"%s", SwigType_str(t,0));
     p = nextSibling(p);
     if (p)
       Printf(out,",");
   }
-  return Swig_temp_result(out);
+  return out;
 }
 
 
